/************************************************
 *			ZPR - praca domowa
 *
 *			Temat: boost::iostreams (devices)
 *
 *			Autor: ukasz Dako	
 *			Grupa: N1-IBERE2
 ************************************************/

/*
		Devices s wprowadzon w bibliotece booost::iostreams koncepcj obiektw zapewniajcych dostp do cigu znakw,
	w zalenoci od typu - wejciowego, wyjciowego albo obu. Wykorzystuje si je do tworzenia obiektw typu 
	stream lub stream_buffer.
		Device realizujcy operacj zapisu nazywa si sink, natomiast realizujcy operacj odczytu nazywa si source.
	Typowo obiekt taki posiada metody: read, write i seek (w zalenoci od typu), oraz definiuje typy char_type i category, na przykad
	prosty source mgby wyglda tak:
*/

#include <iosfwd>                          // streamsize
#include <boost/iostreams/categories.hpp>  // source_tag

using namespace boost::iostreams;

class source_dev {
public:
	// definicja typu znakw, jaki jest przez Device odczytywany
	// najczciej jest to char
	typedef char        char_type;
	// kategoria Device'a - okrela jakie operacje
	// wejcia/wyjcia s w tej klasie realizowane
	// zdefiniowane w bibliotece kategorie znajduj si w pliku
	// boost/iostreams/categories.hpp
	typedef source_tag  category;

	// Device jest rdem (source), wic implementuje tylko metod read
	std::streamsize read(char* s, std::streamsize n)
	{
		// Odczytaj n znakw ze rda i zapisz je do bufora s
		// zwr liczb wczytanych znakw albo -1 w przypadku EOF
	}

};

/*
	Albo prociej, dziedziczc po jednym z dostpnych
	w bibliotece boost::iostreams podstawowych Device'w
	(znajduj si one w pliku boost/iostreams/concepts.hpp):
*/

#include <boost/iostreams/concepts.hpp>  // source

class source_dev_2 : public source {
public:
	std::streamsize read(char* s, std::streamsize n);
};

//******************************************************//
/*
	Pisanie wasnych Device'w daje szerokie moliwoci, np. 
	mona stworzy Device zapisujcy do kontenera STL
*/

#include <algorithm>                       // copy, min
#include <iosfwd>                          // streamsize
#include <boost/iostreams/categories.hpp>  // sink_tag

template<typename Container>
class container_sink {
public:
    typedef typename Container::value_type  char_type;
    typedef sink_tag                        category;
    container_sink(Container& container) : container_(container) { }
    std::streamsize write(const char_type* s, std::streamsize n)
    {
        container_.insert(container_.end(), s, s + n);
        return n;
    }
    Container& container() { return container_; }
private:
    Container& container_;
};

/*
	Biblioteka boost::iostreams udostpnia take gotowe Device, np. obsugujce operacje na plikach,
*/

#include <ostream>
#include <iostream>
#include <vector>
#include <boost/iostreams/device/file.hpp>
#include <boost/iostreams/stream.hpp>

using namespace std;

int main()
{
	// Tworzenie stumienia zapisu do pliku:
	{
		stream_buffer<file_sink> out_buf("HelloWorld.txt");
		ostream                     file_out(&out_buf);
		file_out << "Hello World!" << endl;	// zapisuje do pliku HelloWorld.txt
	}	// faktyczny zapis do pliku dokonuje si dopiero w momencie niszczenia obiektu file_out
		// w tym przypadku po opuszczeniu powyszego bloku kodu

	string str;
	// Tworzenie strumienia odczytu z pliku
	stream_buffer<file_source> in_buf("HelloWorld.txt");
	istream							file_in(&in_buf);
	//file_in >> str;			// pobierze pojedycze sowa
	getline(file_in, str);	// pobierze ca lini
	assert(str == "Hello World!");

	string result;
	// Tworzenie strumienia zapisu do string'a
	stream<container_sink<string>>  str_out(result);
	str_out << "Hello World!";	// dopisze do zmiennej result tekst "Hello World!"
	str_out.flush();
	assert(result == "Hello World!");

	return 0;
}



